page 27,132
TITLE CALLER  - SAMPLE ASSEMBLER ROUTINE  (EXMP1S.ASM)
COMMENT *
		PROLOG - MODULE DESCRIPTION
	This program will demonstrate how to write two programs
	that will have to be linked together.  The first program,
	called CALLER will do the following:
 
		CLEAR THE VIDEO MONITOR
		MOVE THE CURSOR TO THE TOP LEFT POSITION (POS 0,0)
		PRINT A MESSAGE PROMPTING FOR A NAME
		WAIT FOR THE USER TO TYPE IN THE NAME
		CLEAR THE VIDEO MONITOR
		CALL THE SECOND ROUTINE TO PRINT THE NAME MANY TIMES
		TERMINATE       *
	EXTRN RECEIVR:FAR	;EXTERNAL SUBROUTINE
SUBTTL DESCRIPTION OF THE STACK SEGMENT
	PAGE
STACK	SEGMENT	PARA STACK 'STACK'
	DB	64 DUP('STACK   ')
STACK	ENDS
SUBTTL DESCRIPTION OF THE DATA WORKAREA
	PAGE
WORKAREA SEGMENT PARA PUBLIC 'DATA'
MAXCHAR	EQU	25		;Size of user response area
PARMS	LABEL	BYTE		;Start of parameter list
BUFSIZE    DB	MAXCHAR		;Limit of number of char
NAMESIZE   DB	?		;Number of chars typed, including the
;				  carriage return
NAMETEXT   DB	MAXCHAR DUP(' ') ;Text of user response
TERMINATOR DB	'$'		;Text delimiter
;				End of parameter list
MESSG	DB	'What is your name?  ' ;Prompting message
	DB	'$'		;End of prompt message
WORKAREA ENDS
SUBTTL DESCRIPTION OF DOS INTERFACES
	PAGE
CSEG	SEGMENT	PARA PUBLIC 'CODE'
START	PROC	FAR
	ASSUME CS:CSEG,DS:WORKAREA,SS:STACK,ES:NOTHING
;
;    At entry:
;	CS=Addr of entry point
;	IP=0, offset to entry point
;	SS=Segment address of stack segment
;	SP=200H, offset into stack of top of stack
;	DS=Segment address of Program Segment Prefix area
;	ES=Same as DS
;
;    At exit:
;	It is the user's responsibility to ensure that:
;	CS=Segment address of Program Segment Prefix area,
;		originally in DS or ES.
;	SS=Same value as at entry
;	(Other regs may be destroyed.)
SUBTTL ESTABLISH ENTRY LINKAGE FROM DOS
	PAGE
;	Set up the stack to contain the proper values so this
;	program can return to DOS.
;
        PUSH    DS              ;Set return segment addr to stack
	SUB	AX,AX		;Clear a reg
	PUSH	AX		;Put zero return addr to stack
;
;	Establish addressability for the data segment.
;
	MOV	AX,WORKAREA	;Get location of workarea
	MOV	DS,AX		;Set segreg base of data workarea
;	(Extra Segment, ES segreg, not used)
SUBTTL CLEAR THE SCREEN, PUT CURSOR AT TOP LEFT CORNER
	PAGE
	CALL	CLS		;Call the clear screen routine
	MOV	DX,0		;DH=0, DL=0 (DH=Row,DL=Column)
	MOV	BH,0		;Screen # to use
	MOV	AH,2		;Function=locate
	INT	10H		;Call BIOS to move cursor
;
SUBTTL DISPLAY A PROMPT MESSAGE TO CONSOLE
	PAGE
;	The following lines of code will call to a DOS routine
;	telling the computer to display a message pointed to
;	by the DS:DX register.  It will print it in the upper left
;	corner because in the previous routine we positioned
;	the cursor to location 0,0.  It will print everything
;	starting at DX until it reaches a "$".  Then it will
;	stop and return control back to this program.  This is
;	done by setting AH to 9 and executing an interrupt
;	21H.
;
	MOV	DX,OFFSET MESSG	;Point to message to print
	MOV	AH,9		;Function=console output
	INT	21H		;Call DOS to display message
;
SUBTTL READ FROM THE KEYBOARD
	PAGE
;	The following lines of code will call to a DOS routine
;	telling the computer to wait for some input from the
;	keyboard, terminated by 'ENTER'.  It will use a parameter list
;	pointed to by DX.  This list is defined by the above structure,
;	called PARMLIST.
;
;	The DOS routine number is 0AH.  You must load the 
;	AH register with 0AH and do an INT 21H
;
	MOV	DX,OFFSET PARMS	;Point to where to put name
;				Read response from keyboard via DOS
	MOV	AH,10		;Function=read keyboard
	INT	21H		;Call DOS to read keyboard
;
	CALL	CLS		;Call clear screen routine
SUBTTL INVOKE AN EXTERNAL SUBROUTINE
	PAGE
;	At this point, this program will transfer control to
;	another program called RECEIVR.  RECEIVR needs to 
;	receive the information in the data location called
;	NAMETEXT.
;
	MOV	AH,0		;Clear high byte of reg
	MOV	AL,NAMESIZE	;Get count of chars in name
	MOV	SI,AX		;Pass char count to subroutine
	MOV	BX,OFFSET NAMETEXT ;Pass offset to name
;
	CALL	RECEIVR
SUBTTL RESTORE THE ENTRY CONDITIONS, RETURN TO DOS
	PAGE
;	For the RETURN to execute properly, the stack
;	must be restored to the point where only the
;	return parameters are still on the stack.
;	Any values saved on the stack must be popped
;	off to restore segregs to the value expected
;	by DOS.
;
	RET			;Far return to DOS
;
;	This changes the CS to point to the Program Segment Prefix
;	area, whose address was originally passed in DS.
;	The IP is set to zero.  At offset of zero in the
;	Program Header is an INT 20H instruction which
;	returns control to DOS.
;
START	ENDP
SUBTTL COMMON CODE LOCAL SUBROUTINE, CLEARS SCREEN
	PAGE
;	This routine is called several times throughout the program.
;	It uses the BIOS call to do a scroll.  To do this, AH must be
;	6 (Routine number), AL must be zero (Zero signifies to clear
;	the window), CX must point to the upper left corner of the
;	window, (here we use 0,0), and DX must point to the lower
;	right corner (we use 24,79).  Also, BH must be the attribute to
;	use in the clear screen. We will use 7, the normal attribute.
;
CLS	PROC	NEAR
	MOV	CX,0		;Upper left coord (0,0)
	MOV	DX,2479H	;Bottom right position
	MOV	BH,7		;Normal attribute for CLS
;				;Clear entire screen
	MOV	AX,600H		;AH=6 requests scroll, AL=0 means entire window
	INT	10H		;Call BIOS to scroll window
;
	RET			;Return to caller
CLS	ENDP
CSEG	ENDS
	END	START
